/*  "Alert Button" V1.0 (sends a Blynk notification if the button is pressed).
    Copyright (C) 2024  D. Thorpe (unusualelectronics.co.uk)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as published
    by the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#define BLYNK_TEMPLATE_ID "your template id"
#define BLYNK_TEMPLATE_NAME "your template name"
#define BLYNK_AUTH_TOKEN "your token"
/* Comment out line below to disable prints and save space */
//#define BLYNK_PRINT Serial

#include <ESP8266WiFi.h>			// ESP8266 Core WiFi Library

#include <BlynkSimpleEsp8266.h>		// Install "Blynk" Library 1.3.2 or later by Volodymyr Shymanskyy

//--------debugging-----------
// Change Debug to 1 in line below to see status messages
#define DEBUG 0

#if DEBUG == 1
#define debug(x) Serial.print(x)
#define debugln(x) Serial.println(x)
#else
#define debug(x)
#define debugln(x)
#endif
//-------------------

//WiFi credentials.

const char * ssid = "your wi-fi ssid";
const char * password = "your wi-fi password";

const unsigned long connectionTimeout = 15000; // Wi-Fi connection Timeout in milliseconds (15 seconds)

unsigned long connectionStartTime;

int PowerPin = 2; // GPIO pin 2 is used to control power to the ESP module

int beepPin = 4; // GPIO pin 4 is used for the beeper

ADC_MODE(ADC_VCC); // set adc mode to read esp supply voltage
float voltage = ESP.getVcc() / 1000.00; // this will be used to read adc and divide by 1000 to show actual ESP regulator voltage
//=================================================
void setup() {

  pinMode(PowerPin, OUTPUT); // set Powerpin as output 
  digitalWrite(PowerPin, HIGH); // set initial power pin status to HIGH (keep module turned-on)

  pinMode(beepPin, OUTPUT); // Set beeper pin as an output
  digitalWrite(beepPin, LOW); // set initial BEEPER pin status to LOW (off) (should already be low - just make sure)

  Serial.begin(115200);
  debugln(".........");

  connectionStartTime = millis();
  debug(millis());
  debugln(" Connecting to Wi-Fi...");
  WiFi.begin(ssid, password);
  Blynk.config(BLYNK_AUTH_TOKEN); // configure bylnk to connect when Blynk.run command is used
  tone(beepPin, 1400); // beep at start
  delay(50); // beep duration
  noTone(beepPin); // turn off the beeper

  Serial.println(voltage);
}
//=================================================

void loop() {

  // check WiFi connection status:

  if ((WiFi.status() != WL_CONNECTED) && millis() - connectionStartTime >= connectionTimeout) {
    debugln(millis());
    debugln(" Wi-Fi Connection timeout!");
    shutdown(); //Shutdown the module due to no Wi-Fi connection
  }
  //--------------------------------------------------------
  else {

    // Do stuff when wifi connected
    if (WiFi.status() == WL_CONNECTED) {

      debug(millis());
      debugln(" Connected to Wi-Fi");
      debugln(WiFi.localIP());

      //========================================================

      if (Blynk.connected()) {
        debug(millis());
        debugln(" Blynk connected ok");

        Blynk.run();
        debug(millis());
        delay(100); //wait 100ms 
        debugln(" Sending Blynk event");
        Blynk.logEvent("button_press");
        Blynk.virtualWrite(V0, voltage); // for supply voltage

        if (voltage <= 3.00) {
          delay(1000); //wait 1000ms
          debugln(" Sending low voltage event");
          Blynk.logEvent("voltage_low", "Battery voltage is low");
        }

        delay(100); //wait 100ms
        Blynk.disconnect();
        delay(100); //wait 100ms
        debug(millis());
        debugln(" Blynk event notification completed OK");
        //  play tune to indicate success
        tone(beepPin, 1568); // beep note
        delay(300); // beep duration
        noTone(beepPin); // turn off the beeper
        delay(50); // silenceduration
        tone(beepPin, 1760); // beep note
        delay(300); // beep duration
        noTone(beepPin); // turn off the beeper
        delay(50); // silence duration
        tone(beepPin, 1397); // beep note
        delay(300); // beep duration
        noTone(beepPin); // turn off the beeper
        delay(50); // silence duration
        tone(beepPin, 699); // beep note
        delay(300); // beep duration
        noTone(beepPin); // turn off the beeper
        delay(50); // silence duration
        tone(beepPin, 523); // beep note
        delay(600); // beep duration
        noTone(beepPin); // turn off the beeper

        shutdown(); //Shutdown the module now that blynk has finished
      }
      //------------------------------
      else
        debug(millis());
      debugln(" Connecting to Blynk");
      Blynk.connect();
      debugln(millis());

      if (Blynk.connected() == false) {
        debug(millis());
        debugln(" Blynk connection failed");
        shutdown(); //Shutdown the module 

      }
    }
  }
}
//=================================================

void shutdown() {
  debug(millis());
  debugln(" Module is now turning OFF");
  digitalWrite(PowerPin, LOW); // turn module power off
  while (true) {
    ESP.wdtFeed(); //prevent watchdog timer reset during loop
  } //loop here forever while power is still on
}